/* O lista de numere intregi implementata ca tablou */

class ListaTI {
  private int tab[]; // tabloul care contine elementele listei
  private int lungime; // lungimea listei

  public ListaTI(int capacitate) {
    if(capacitate<2) capacitate=2;
    tab=new int[capacitate];
    lungime=0;
  }

  public int capacitate() {
    return tab.length; // capacitatea listei este lungimea tabloului
  }

  public int lungime() {
    return lungime; // lungimea listei este valoarea campului lungime
  }

  /* se adauga un element la sfarsitul listei */
  public boolean adauga(int elem) {
    if(lungime==tab.length) return false;
    tab[lungime]=elem;
    lungime++;
    return true;
  }

  /* Se insereaza un element pe pozitia indice */
  public boolean insereaza(int indice, int elem) {
    if(lungime==tab.length || indice<0) return false;
    if(indice>=lungime) return adauga(elem);
    /* deplasarea spre dreapta a elementelor */
    for(int i=lungime; i>indice; i--) tab[i]=tab[i-1];
    /* inserarea noului element */
    tab[indice]=elem;
    lungime++;
    return true;
  }

  /* Se elimina elementul de pe pozitia indice */
  public boolean elimina(int indice) {
    if(indice<0 || indice>=lungime) return false;
    /* deplasarea la stanga a elementelor care urmeaza */
    for(int i=indice+1; i<lungime; i++) tab[i-1]=tab[i];
    lungime--;
    return true;
  }

  /* Intoarce elementul de pe pozitia indice */
  public int elementLa(int indice) throws Exception {
    if(indice<0 || indice>=lungime)
      throw new Exception("ListaTD: indice gresit "+indice);
    return tab[indice];
  }

  /* Se cauta un element in lista. Intoarce indicele elementului
     sau -1 daca nu exista
  */
  public int cauta(int elem) {
    for(int i=0; i<lungime; i++)
      if(tab[i]==elem) return i;
    return -1;
  }

  /* Se pune o noua valoare unui element deja existent */
  public boolean pune(int indice, int elem) {
    if(indice<0 || indice>=lungime) return false;
    tab[indice]=elem;
    return true;
  }

  /* Se converteste lista in sir */
  public String toString() {
    StringBuffer b=new StringBuffer("[");
    if(lungime>0) {
      for(int i=0; i<lungime-1; i++) b.append(tab[i]+", ");
      b.append(tab[lungime-1]);
    }
    b.append("]");
    return b.toString();
  }
} // Sfarsit clasa ListaTI


  /* Testarea clasei ListaTD */
class TestListaTI {
  public static void main(String args[]) {
    ListaTI lista=new ListaTI(10);
    System.out.println("Lista nou creata: "+lista);
    lista.adauga(11);
    lista.adauga(22);
    lista.adauga(33);
    lista.adauga(44);
    System.out.println("Dupa patru adaugari: "+lista);
    lista.insereaza(2,-77);
    System.out.println("Dupa o inserare: "+lista);
    lista.elimina(3);
    System.out.println("Dupa o eliminare: "+lista);
    System.out.println("Elementul -77 se gaseste la indicele: "+
      lista.cauta(-77));
    lista.pune(2, 25);
    System.out.println("Dupa inlocuirea unui element: "+lista);
  }
}
