public class ArboreSelectie {
  private Entry tab[]; // tabloul de intrari
  private int size, increment; // numarul de elemente si capacitatea

  class Entry { // clasa perechilor (prioritate, element)
    int prioritate;
    Object element;

    Entry(Object obj, int prior) { // constructorul clasei interioare
      prioritate=prior;
      element=obj;
    }

    public String toString() { // redefinirea metodei din clasa Object
      return "("+prioritate+" "+element.toString()+")";
    }
  } // sfarsit clasa Entry

  /* Constructorii clasei ArboreSelectie */

  /* Se construieste un arbore de selectie vid, cu capacitatea
     implicita 100 si incrementul implicit 50
  */
  ArboreSelectie() { 
    tab=new Entry[100];
    size=0;
    increment=50;
  }

  /* Se construieste un arbore de selectie vid cu capacitatea
     si incrementul date
  */
  ArboreSelectie(int initialCapacity, int capacityIncrement) {
    tab=new Entry[initialCapacity];
    size=0;
    increment=capacityIncrement;
  }

  /* Metodele private ale clasei ArboreSelectie */

  /* Intoarce indicele fiului stanga al nodului de indice i */
  private int fiuStanga(int i) {
    int j=2*i+1;
    if(j>size-1) j=-1;
    return j;
  }

  /* intoarce indicele fiului dreapta al nodului de indice i */
  private int fiuDreapta(int i) {
    int j=2*i+2;
    if(j>size-1) j=-1;
    return j;
  }

  /* Intoarce indicele tatalui nodului de indice i */
  private int tata(int j) {
    if(j==0) return -1;
    return (j-1)/2;
  }

  /* Inlocuieste tabloul vechi cu unul nou, avand capacitate mai mare
     si copiaza elementele din tabloul vechi in cel nou
  */
  private void maresteCapacitate() {
    Entry tab1[]=new Entry[tab.length+increment];
    for(int i=0; i<size; i++) tab1[i]=tab[i];
    tab=tab1;
  }

  /* Intoarce indicele fiului cu prioritate maxima al nodului tata.
     Daca nu are fii, intoarce -1;
  */
  private int fiuMaxim(int tata) {
    int fiuAles=-1, fiuSt, fiuDr;
    fiuSt=fiuStanga(tata);
    fiuDr=fiuDreapta(tata);
    if(fiuSt>0) {
      if(fiuDr>0) 
         fiuAles=(tab[fiuSt].prioritate>tab[fiuDr].prioritate)?
              fiuSt:fiuDr;
      else fiuAles=fiuSt;
    } else if(fiuDr>0) fiuAles=fiuDr;
    return fiuAles;
  }


  /* Metodele publice ale clasei ArboreSelectie */
  public int size() { // intoarce numarul de noduri
    return size;
  }

  public int capacity() { // intoarce capacitatea tabloului
    return tab.length;
  }

  /* Punerea in arbore a unei perechi (prioritate, element) */
  public void put(Object obj, int priority) {
    int inou, itata;
    boolean stop=false;
    Entry aux;
    if(size==tab.length) maresteCapacitate();
    inou=size;
    tab[inou]=new Entry(obj, priority);
    size++;
    itata=tata(inou);
    while(itata>=0 && !stop) {
      if(tab[itata].prioritate<tab[inou].prioritate) {
        aux=tab[itata];
        tab[itata]=tab[inou];
        tab[inou]=aux;
        inou=itata;
        itata=tata(inou);
      }
      else stop=true;
    }
  }

  /* Extragerea unui element din arborele de selectie */
  public Object remove() {
    if(size==0) return null;
    Object element=tab[0].element;
    Entry aux;
    boolean stop=false;
    int fiuAles, tata;
    if(size>1) {
      tab[0]=tab[size-1];
      tab[size-1]=null;
      tata=0;
      size--;
      do {
        fiuAles=fiuMaxim(tata);
        if(fiuAles>0 && tab[fiuAles].prioritate>tab[tata].prioritate){
          aux=tab[tata];
          tab[tata]=tab[fiuAles];
          tab[fiuAles]=aux;
          tata=fiuAles;
        } else stop=true;
      } while(!stop);
    } else size--;
    return element;
  }

  /* Intoarce primul element, fara sa-l extraga */
  public Object get() {
    if(size>0) return tab[0].element;
    else return null;
  }

  /* Conversia arborelui de selectie in sir */
  public String toString() {
    String str="[";
    for(int i=0; i<size-1; i++) str=str+tab[i].toString()+" ";
    if(size>0) str=str+tab[size-1];
    return str+"]";
  }    

}
