import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;

public class TestJTree2 {
  GUI gui=new GUI();
  JTree tree;
  DefaultMutableTreeNode radacina;
  TreePath path;
  TreeSelectionListener ascultSelectie;

  class GUI extends JFrame {
    JLabel lab;
    JButton pune, elimina, preordine, postordine, latime, stergere;
    JPanel panouButoane;
    JScrollPane sp1, sp2;
    Container cp;
    JSplitPane splitPane;
    JTextArea textArea;
    
    GUI() {
      super("Testarea clasei JTree");
      cp=getContentPane();
      /* Se creeaza nodul radacina si instanta initiala a clasei
         JTree, careia i se adauga ascultatorul de selectie
      */
      radacina=new DefaultMutableTreeNode("Radacina arborelui");
      tree=new JTree(radacina);
      ascultSelectie=new AscultSelectie();
      tree.addTreeSelectionListener(ascultSelectie);
      /* Se pune arborele intr-un panou de defilare (JScrollPane)
         care se pune in prima parte a panoului impartit in doua
         (JSplitPane)
      */
      sp1=new JScrollPane(tree);
      splitPane=new JSplitPane();
      splitPane.setTopComponent(sp1);
      /* Se creeaza aria de text (JTextArea) si se pune intr-un
         JScrollPane, care se pune in partea a doua a panoului
         divizat
      */
      textArea=new JTextArea();
      textArea.setLineWrap(true);
      textArea.setWrapStyleWord(true);
      sp2=new JScrollPane(textArea);
      splitPane.setRightComponent(sp2);
      cp.add(splitPane, BorderLayout.CENTER);
      /* Se creeaza butoanele si li se adauga ascultatoarele
         de actiune corespunzatoare
      */
      pune=new JButton("Pune fiu");
      pune.addActionListener(new PuneFiu());
      elimina=new JButton("Elimina nod");
      elimina.addActionListener(new EliminaNod());
      preordine=new JButton("In preordine");
      postordine=new JButton("In postordine");
      latime=new JButton("In latime");
      stergere=new JButton("Stergere text");
      Traversari traversari=new Traversari();
      preordine.addActionListener(traversari);
      postordine.addActionListener(traversari);
      latime.addActionListener(traversari);
      stergere.addActionListener(new StergereText());
      /* Se creeaza panoul de butoane si se adauga butoanele
      */
      panouButoane=new JPanel();
      panouButoane.add(pune);
      panouButoane.add(elimina);
      panouButoane.add(preordine);
      panouButoane.add(postordine);
      panouButoane.add(latime);
      panouButoane.add(stergere);
      cp.add(panouButoane, BorderLayout.NORTH);
      /* Se crreaza eticheta de la partea inferioara (pentru mesaje)
      */
      lab=new JLabel();
      cp.add(lab, BorderLayout.SOUTH);
      /* Se finalizeaza construirea interfetei grafice
      */
      addWindowListener(new Stop());
      setLocation(50, 50);
      setSize(700, 500);
      setVisible(true);
    }
  }

  /* Clasa ascultatoare a evenimentelor de selectare a nodurilor
  */
  class AscultSelectie implements TreeSelectionListener {
    public void valueChanged(TreeSelectionEvent e) {
      path=e.getPath(); // se memoreaza calea la nodul selectat
      gui.lab.setText("Calea selectata: "+path.toString());
    }
  }

  /* Clasa ascultatoare de evenimente pentru punerea unui nod
  */
  class PuneFiu implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      if(path==null) {
        gui.lab.setText("Nici un nod nu este selectat");
        return;
      }
      /* Nodul selectat este ultimul din calea catre acesta
      */      
      DefaultMutableTreeNode nodSelectat=
        (DefaultMutableTreeNode)path.getLastPathComponent();
      /* Se cere utilizatorului numele noului nod 
      */
      String input= JOptionPane.showInputDialog(gui,
                             "Dati numele noului nod");
      if(input!=null) 
        if(input.equals(""))
          JOptionPane.showMessageDialog(gui,
                   "Numele nodului nu poate fi vid");
        else {
          /* Se creeaza si se adauga nodul nou
          */
          DefaultMutableTreeNode nodNou=
             new DefaultMutableTreeNode(input);
          nodSelectat.add(nodNou);
          /* Se creeaza noua instanta a clasei JTree si se pune 
             in locul celei vechi, adaugandu-i-se si ascultatorul
             de evenimente de selectie a nodurilor
          */
          tree=new JTree(radacina);
          tree.addTreeSelectionListener(ascultSelectie);
          gui.sp1=new JScrollPane(tree);
          gui.splitPane.setTopComponent(gui.sp1);
          /* Se determina calea catre nodul nou si se face vizibila
          */
          TreeNode cale[]=nodNou.getPath();
          TreePath path1=new TreePath(cale);
          tree.makeVisible(path1);
          gui.lab.setText("La nodul "+nodSelectat+" s-a pus fiul "+
             nodNou);
        }
    }
  }

  class EliminaNod implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      TreePath path=tree.getSelectionPath();
      if(path==null) gui.lab.setText("Nu este nici un nod selectat");
      else {
        /* Se determina nodul de eliminat, ca fiind ultimul
           din calea catre el
        */
        DefaultMutableTreeNode nodEliminat=
          (DefaultMutableTreeNode)path.getLastPathComponent();
        if(nodEliminat==radacina)
          JOptionPane.showMessageDialog(gui,
            "Nu se poate elimina radacina arborelui");
        else {
          /* Se elimina nodul selectat
          */
          nodEliminat.removeFromParent();
          /* Se creeaza o noua instanta a clasei JTree si se pune in
             locul celei vechi
          */
          tree=new JTree(radacina);
          tree.addTreeSelectionListener(ascultSelectie);
          gui.sp1=new JScrollPane(tree);
          gui.splitPane.setTopComponent(gui.sp1);
          gui.lab.setText("eliminat nodul "+nodEliminat);
        }
      }
    }
  }

  /* Clasa ascultatoare pentru traversarile arborelui
  */
  class Traversari implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      Enumeration enum=null;;
      String actiune=e.getActionCommand();
      /* Se stabileste tipul traversarii si se creeaza enumeratia
         corespunzatoare (se foloseste interfata
           java.util.Enumeration)
      */
      if(actiune.equals("In preordine")) {
        enum=radacina.preorderEnumeration();
        gui.textArea.append("Traversare in preordine: ");
      } else if(actiune.equals("In postordine")) {
          enum=radacina.postorderEnumeration();
          gui.textArea.append("Traversare in postordine: ");
      } else if(actiune.equals("In latime")) {
        enum=radacina.breadthFirstEnumeration();
        gui.textArea.append("Traversare in latime: ");
      }
      /* Se pun in aria de text nodurile arborelui in ordinea
         data de enumeratie
      */
      while(enum.hasMoreElements())
        gui.textArea.append("("+enum.nextElement()+") ");
      gui.textArea.append("\n");
    }
  }

  class StergereText implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      gui.textArea.setText("");
    }
  }

  class Stop extends WindowAdapter {
    public void windowClosing(WindowEvent e) {
      System.exit(0);
    }
  }

  public static void main(String args[]) {
    TestJTree2 test=new TestJTree2();
  }
}